/**************************************************************************
*   Copyright (C) 2005-2006 by Jinsoo Kang                                *
*   jsk@gravitybind.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "tx.cfg"
#include "tx.h"
#include "tx_util.h"
#include "helpbrowser.h"
#include "abouttxdialog.h"

#include <math.h>
#include <iostream>
#include <cstdlib>

using namespace std;


Q3IntDict<QPixmap> Tx::messageIcons = Q3IntDict<QPixmap>();
QString Tx::externalEditor;
bool Tx::externalEditorActive = TRUE;
int Tx::numRecentDocs = 20;

QStringList Tx::recentDocList;


Tx::Tx() : Q3MainWindow( 0, "tX", Qt::WDestructiveClose ) {
	messageIcons.setAutoDelete(TRUE);
	if (messageIcons.isEmpty()) for (int i = (int)QMessageBox::NoIcon; i <= (int)QMessageBox::Question; i++) {
		QImage currImg = QMessageBox::standardIcon( (QMessageBox::Icon)i ).convertToImage();
		if (currImg.isNull()) continue;
		QPixmap* currPix = new QPixmap( currImg.smoothScale(18,18) );
		messageIcons.insert(i, currPix);
	}

	printer = new QPrinter();

	QPixmap appIcon = QPixmap(":images/tXsmall.png");
	setIcon(appIcon);

	QSplitter *splitter = new QSplitter(this);
	splitter->setOrientation(Qt::Vertical);
	splitter->setChildrenCollapsible(TRUE);
	setCentralWidget( splitter );

	xtable = new XmlTable(500, 20, splitter);
	bottomTabs = new QTabWidget(splitter);
	bottomTabs->setTabPosition(QTabWidget::Bottom);

	QFont editfont("Courier", 14);
	cellEdit = new Q3TextEdit(bottomTabs);
	cellEdit->setTabStopWidth(4);
	cellEdit->setTextFormat(Qt::PlainText);
	cellEdit->setWordWrap(Q3TextEdit::NoWrap);
	cellEdit->setFont(editfont);
	bottomTabs->addTab(cellEdit, tr("Edit Cell"));
	dtdEdit = new Q3TextEdit(bottomTabs);
	dtdEdit->setTabStopWidth(4);
	dtdEdit->setTextFormat(Qt::PlainText);
	dtdEdit->setWordWrap(Q3TextEdit::NoWrap);
	dtdEdit->setFont(editfont);
	bottomTabs->addTab(dtdEdit, tr("DTD"));
	errorTable = new MessageList();
	bottomTabs->addTab(errorTable, tr("Messages"));
	QObject* errorTableListView = errorTable->child("messageListView", "Q3ListView");
	connect(
		errorTableListView, SIGNAL( selectionChanged(Q3ListViewItem*) ),
		this, SLOT( openMessageTarget(Q3ListViewItem*) )
		);
	connect(
		errorTableListView, SIGNAL( doubleClicked(Q3ListViewItem*, const QPoint&, int) ),
		this, SLOT( openMessageTarget(Q3ListViewItem*) )
		);

	helpText = new HelpBrowser(bottomTabs);
	bottomTabs->addTab(helpText, tr("Help Text"));

	Q3ValueList<int> sizeList;
	sizeList.append(1000);
	sizeList.append(10);
	splitter->setSizes(sizeList);

	createFileMenu();
	createEditMenu();
	createFormatMenu();
	createHelpMenu();
	createStatusBar();

	connect( cellEdit, SIGNAL(textChanged()), this, SLOT(updateCellText()) );
	connect( cellEdit, SIGNAL(modificationChanged(bool)), xtable, SLOT(touch(bool)) );
	connect( dtdEdit, SIGNAL(modificationChanged(bool)), xtable, SLOT(touch(bool)) );
	connect( xtable, SIGNAL(dtdLoaded(const QString &)), dtdEdit, SLOT(setText(const QString &)) );
	connect(
		xtable, SIGNAL(message(const QString &, const QString &, bool, bool, QMessageBox::Icon)), 
		this, SLOT(message(const QString &, const QString &, bool, bool, QMessageBox::Icon))
		);

	pref = new PrefDialog( this, tr("Preferences") );
	pref->setCaption( tr("Preferences") );
	connect( pref, SIGNAL(applyButtonPressed()), xtable, SLOT(updateAll()) );
	connect( pref, SIGNAL(applyButtonPressed()), this, SLOT(applyStyleIcons()) );
	connect( pref, SIGNAL(cancelButtonPressed()), xtable, SLOT(updateAll()) );

	message( tr("Ready") );
	applySettings();
}


Tx::~Tx() {
	delete printer;
	delete errorTable;
}


void Tx::openMessageTarget(Q3ListViewItem *item) {
	int line = -1, row = -1, col = -1;
	QString target = item->text(0);
	QStringList pairs = QStringList::split(',', target);
	for (QStringList::Iterator it = pairs.begin(); it != pairs.end(); ++it) {
		QString key, val;
		QStringList curr = QStringList::split(':', *it);
		QStringList::Iterator it2 = curr.begin();
		if (it2 != curr.end()) key = *it2;
		++it2;
		if (it2 != curr.end()) val = *it2;
		key = key.stripWhiteSpace();
		val = val.stripWhiteSpace();

		if (!val.isEmpty()) {
			if (key == tr("Line")) line = val.toInt();
			else if (key == tr("Row")) row = val.toInt();
			else if (key == tr("Column")) col = val.toInt();
		}
	}
	if (line >= 0 && Tx::externalEditorActive) {
		Q3Process proc(Tx::externalEditor);
		proc.addArgument(filename);
		proc.launch(QString(""));
	}
	else if (row > 0 && col > 0) {
		xtable->clearSelection();
		row--; col--;
		xtable->setCurrentCell(row, col);
	}
}


void Tx::message(const QString &msg, const QString &loc, bool printStatusBar, bool printLog, QMessageBox::Icon icn) {
	if (printLog) {
		QObject *list = errorTable->child("messageListView", "Q3ListView");
		if (list == NULL) return;
		Q3ListView *messageListView = (Q3ListView*) list;
		Q3ListViewItem *currItem = new Q3ListViewItem(messageListView, loc, msg);
		QPixmap *currPix = messageIcons[icn];
		if (currPix != NULL) currItem->setPixmap( 1, *currPix );
		messageListView->insertItem(currItem);
		messageListView->scrollBy(0, 100);
		bottomTabs->showPage(errorTable);
	}
	if (printStatusBar) {
		statusBar()->message( msg, 5000 );
	}
}


void Tx::clearMessages() {
	QObject *list = errorTable->child("messageListView", "Q3ListView");
	if (list == NULL) return;
	Q3ListView *messageListView = (Q3ListView*) list;
	messageListView->clear();
}



void Tx::applySettings() {
	QSettings settings;
	settings.setPath(TX_SETTINGS_PATH, "tx", QSettings::UserScope);
	settings.beginGroup("/tx");

	int xpos = settings.readNumEntry( "/x", 0 );
	int ypos = settings.readNumEntry( "/y", 30 );
	int width = settings.readNumEntry( "/w", 650 );
	int height = settings.readNumEntry( "/h", 600 );
	resize(width, height);
	move(xpos, ypos);

	recentDocList.clear();
	QStringList list = settings.readListEntry("/recentdocs");
	QStringList::Iterator it = list.begin();
	while( it != list.end() ) {
		recentDocList.push_back(*it);
		++it;
	}
	settings.endGroup();
}


void Tx::saveSettings() {
	QSettings settings;
	settings.setPath(TX_SETTINGS_PATH, "tx", QSettings::UserScope);
	settings.beginGroup("/tx");

	settings.writeEntry( "/x", x() );
	settings.writeEntry( "/y", y() );
	settings.writeEntry( "/w", width() );
	settings.writeEntry( "/h", height() );

	settings.writeEntry( "/recentdocs", recentDocList );
	settings.endGroup();
}


void Tx::updateCellText() {
	int currRow = xtable->currentRow();
	int currCol = xtable->currentColumn();
	if (currRow < 0 || currRow >= xtable->numRows()) return;
	if (currCol < 0 || currCol >= xtable->numCols()) return;
	QString oldstr = xtable->text(currRow, currCol);
	QString newstr = cellEdit->text();
	if (oldstr.isNull()) oldstr = "";
	if (newstr.isNull()) newstr = "";
	if (oldstr != newstr) {
		xtable->addUndoRecord(currRow, currCol, currRow, currCol);
	}
	xtable->ensureCellVisible(currRow, currCol);
	xtable->setText(currRow, currCol, cellEdit->text());
	xtable->updateCell(currRow, currCol);
}


void Tx::createFileMenu() {
	fileMenu = new Q3PopupMenu( this );
	menuBar()->insertItem( tr("&File"), fileMenu );
	fileMenu->insertItem(QIcon(":images/file.png"), tr("&New"), this, SLOT(newDoc()), Qt::CTRL+Qt::Key_N );
	QIcon openIcon(":images/open.png");
	fileMenu->insertItem(openIcon, tr("&Open..."), this, SLOT(choose()), Qt::CTRL+Qt::Key_O );

	recentMenu = new Q3PopupMenu(this, "recentDocMenu");
	connect( recentMenu, SIGNAL(aboutToShow()), this, SLOT(updateRecentDocMenu()) );
	connect( recentMenu, SIGNAL(activated(int)), this, SLOT(openRecentFile(int)) );
	fileMenu->insertItem(openIcon, tr("Open Recent"), recentMenu );

	openAsTextMenuId = fileMenu->insertItem(openIcon, tr("Open As Text"), this, SLOT(openAsText()) );

	reloadMenuId = fileMenu->insertItem(QIcon(":images/refresh.png"),  tr("&Reload"), this, SLOT(reload()), Qt::CTRL+Qt::Key_L );

	fileMenu->insertSeparator();
	QIcon saveIcon(":images/save.png");
	fileMenu->insertItem(saveIcon, tr("&Save"), this, SLOT(save()), Qt::CTRL+Qt::Key_S );
	fileMenu->insertItem(saveIcon, tr("Save &As..."), this, SLOT(saveAs()), Qt::CTRL+Qt::SHIFT+Qt::Key_S );
	fileMenu->insertItem(QIcon(":images/convert.png"), tr("&Export to HTML..."), this, SLOT(exportFile()), Qt::CTRL+Qt::Key_E );
	fileMenu->insertSeparator();
	fileMenu->insertItem(QIcon(":images/print.png"), tr("&Print..."), this, SLOT(print()), Qt::CTRL+Qt::Key_P );
	fileMenu->insertSeparator();
	fileMenu->insertItem(QIcon(":images/close.png"), tr("&Close"), this, SLOT(close()), Qt::CTRL+Qt::Key_W );
	fileMenu->insertItem(QIcon(":images/quit.png"), tr("&Quit"), qApp, SLOT( closeAllWindows() ), Qt::CTRL+Qt::Key_Q );

	fileMenu->setItemEnabled(openAsTextMenuId, FALSE);
	fileMenu->setItemEnabled(reloadMenuId, FALSE);
	connect( fileMenu, SIGNAL(aboutToShow()), this, SLOT(updateFileMenu()) );
}



void Tx::updateFileMenu() {
	bool isEnabled = !this->filename.isEmpty();
	fileMenu->setItemEnabled(openAsTextMenuId, isEnabled && Tx::externalEditorActive);
	fileMenu->setItemEnabled(reloadMenuId, isEnabled);
}


void Tx::openAsText() {
	if (Tx::externalEditorActive) {
		Q3Process proc(Tx::externalEditor);
		proc.addArgument(filename);
		proc.launch(QString(""));
	}
}


void Tx::updateRecentDocList(const QString& newitem) {
	if (!newitem.isEmpty()) {
		recentDocList.remove(newitem);
		recentDocList.push_back(newitem);
		while (recentDocList.size() > Tx::numRecentDocs) recentDocList.pop_front();
	}
}


void Tx::updateRecentDocMenu() {
	if (recentMenu == NULL) return;
	QStringList::Iterator it = recentDocList.begin();
	for ( int i = 0; it != recentDocList.end() && i < Tx::numRecentDocs; i++ ) {
		if (recentMenu->idAt(i) < 0) {
			recentMenu->insertItem(*it, i, i);
		}
		else {
			recentMenu->changeItem(i, *it);
		}
		recentMenu->setItemParameter(i, i);
		++it;
	}
}


void Tx::openRecentFile(int id) {
	Q_ASSERT(recentDocList.count() > id);
	QString fn = recentDocList[id];
	load( recentDocList[id] );
}


void Tx::createEditMenu() {
	editMenu = new Q3PopupMenu( this );
	menuBar()->insertItem( tr("&Edit"), editMenu );
	undoMenuId = editMenu->insertItem(QIcon(":images/undo.png"), tr("&Undo"), xtable, SLOT(undo()), Qt::CTRL+Qt::Key_Z );
	redoMenuId = editMenu->insertItem(QIcon(":images/redo.png"), tr("Re&do"), xtable, SLOT(redo()), Qt::CTRL+Qt::SHIFT+Qt::Key_Z );
	editMenu->insertSeparator();
	editMenu->insertItem(QIcon(":images/cut.png"), tr("Cu&t"), this, SLOT(cut()), Qt::CTRL+Qt::Key_X );
	QIcon copyIcon(":images/copy.png");
	editMenu->insertItem(copyIcon, tr("&Copy"), this, SLOT(copy()), Qt::CTRL+Qt::Key_C );
	editMenu->insertItem(copyIcon, tr("&Copy As Text"), this, SLOT(copyAsText()) );
	editMenu->insertItem(copyIcon, tr("&Copy As XML"), xtable, SLOT(copyXml()), Qt::SHIFT+Qt::CTRL+Qt::Key_C );
	editMenu->insertSeparator();
	QIcon pasteIcon(":images/paste.png");
	editMenu->insertItem(pasteIcon, tr("&Paste"), this, SLOT(paste()), Qt::CTRL+Qt::Key_V );
	editMenu->insertItem(pasteIcon, tr("&Paste As XML"), xtable, SLOT(pasteXml()), Qt::SHIFT+Qt::CTRL+Qt::Key_V );
	editMenu->insertSeparator();
	editMenu->insertItem(QIcon(":images/delete.png"), tr("&Delete"), this, SLOT(del()), Qt::Key_Delete );
	editMenu->insertSeparator();
	editMenu->insertItem(QIcon(":images/selectall.png"), tr("Select &All"), this, SLOT(selectAll()), Qt::CTRL+Qt::Key_A );
	editMenu->insertSeparator();
	editMenu->insertItem(QIcon(":images/find.png"), tr("&Find"), this, SLOT(find()), Qt::CTRL+Qt::Key_F );
	editMenu->insertItem(QIcon(":images/findnext.png"), tr("Find &Next"), this, SLOT(findNext()), Qt::CTRL+Qt::Key_G );
	editMenu->insertItem(QIcon(":images/findprevious.png"), tr("Find Pre&vious"), this, SLOT(findPrevious()), Qt::SHIFT+Qt::CTRL+Qt::Key_G );
	editMenu->insertItem(QIcon(":images/replace.png"), tr("&Replace"), this, SLOT(replace()), Qt::CTRL+Qt::Key_R );
	editMenu->insertSeparator();
	editMenu->insertItem(QIcon(":images/dtd.png"), tr("Document Type"), this, SLOT(docType()));
	editMenu->insertSeparator();

	QIcon ico = QIcon(":images/preferences.png");
	editMenu->insertItem(ico, tr("Pr&eferences..."), this, SLOT(preferences()) );

	connect(xtable, SIGNAL(undoRecordSizeChanged(int)), this, SLOT(updateUndoMenu(int)));
	connect(xtable, SIGNAL(redoRecordSizeChanged(int)), this, SLOT(updateRedoMenu(int)));
	editMenu->setItemEnabled(undoMenuId, FALSE);
	editMenu->setItemEnabled(redoMenuId, FALSE);

	QKeySequence emptyKey;
	findToolBar = new Q3ToolBar( this );
	findToolBar->setOrientation(Qt::Horizontal);
	findToolBar->setLabel( tr("Find") );
	findToolBar->setPaletteBackgroundColor( XmlTable::style(XmlTableItem::EMPTY)->bgColor() );
	findToolBar->setPaletteForegroundColor( XmlTable::style(XmlTableItem::EMPTY)->fgColor() );

	QLabel *findLabel = new QLabel( tr("Find:"), findToolBar, "findLabel" );
	findLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	findEdit = new QLineEdit(findToolBar, "findEdit");
	connect( findEdit, SIGNAL(returnPressed()), this, SLOT(findNext()) );

	QAction *findNextAction = new QAction( tr("Find Next"), this );
	findNextAction->setShortcut(emptyKey);
	findNextAction->addTo(findToolBar);
	connect( findNextAction, SIGNAL(activated()), this, SLOT(findNext()) );

	QAction *findPreviousAction = new QAction( tr("Find Previous"), this );
	findPreviousAction->setShortcut(emptyKey);
	findPreviousAction->addTo(findToolBar);
	connect( findPreviousAction, SIGNAL(activated()), this, SLOT(findPrevious()) );

	findToolBar->addSeparator();
	QAction *matchCaseAction = new QAction( tr("Match case"), this );
	matchCaseAction->setShortcut(emptyKey);
	matchCaseAction->setToggleAction(TRUE);
	matchCase = TRUE;
	matchCaseAction->setOn(matchCase);
	matchCaseAction->addTo(findToolBar);
	connect( matchCaseAction, SIGNAL(toggled(bool)), this, SLOT(setMatchCase(bool)) );


	replaceToolBar = new Q3ToolBar( this );
	replaceToolBar->setOrientation(Qt::Horizontal);
	replaceToolBar->setLabel( tr("Replace") );
	replaceToolBar->setPaletteBackgroundColor( XmlTable::style(XmlTableItem::EMPTY)->bgColor() );
	replaceToolBar->setPaletteForegroundColor( XmlTable::style(XmlTableItem::EMPTY)->fgColor() );

	QLabel *replaceLabel = new QLabel( tr("Replace:"), replaceToolBar, "replaceLabel" );
	replaceLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	replaceEdit = new QLineEdit(replaceToolBar, "replaceEdit");
	connect( replaceEdit, SIGNAL(returnPressed()), this, SLOT(replaceNext()) );

	QFontMetrics metric(replaceLabel->font());
	int labelWidth = metric.width(tr(" Replace:"));
	findLabel->setMinimumWidth(labelWidth);
	replaceLabel->setMinimumWidth(labelWidth);

	QAction *replaceAction = new QAction( tr("Replace"), this );
	replaceAction->setShortcut(emptyKey);
	replaceAction->addTo(replaceToolBar);
	connect( replaceAction, SIGNAL(activated()), this, SLOT(replaceNext()) );
	QAction *replaceAllAction = new QAction( tr("Replace All"), this );
	replaceAllAction->setShortcut(emptyKey);
	replaceAllAction->addTo(replaceToolBar);
	connect( replaceAllAction, SIGNAL(activated()), this, SLOT(replaceAll()) );


	findToolBar->setCloseMode(Q3DockWindow::Always);
	replaceToolBar->setCloseMode(Q3DockWindow::Always);
	moveDockWindow(findToolBar, Qt::DockBottom);
	moveDockWindow(replaceToolBar, Qt::DockBottom);
	findToolBar->hide();
	replaceToolBar->hide();
	connect( findToolBar, SIGNAL(visibilityChanged(bool)), this, SLOT(hideReplaceToolBar(bool)) );
}


void Tx::updateUndoMenu(int size) {
	editMenu->setItemEnabled(undoMenuId, size > 0);
}


void Tx::updateRedoMenu(int size) {
	editMenu->setItemEnabled(redoMenuId, size > 0);
}


void Tx::hideReplaceToolBar(bool vis) {
	if (!vis) replaceToolBar->hide();
}



void Tx::applyStyleIcons() {
	applyStyleIcons(XmlTableItem::ELEMENT, tr("element"), actionElement);
	applyStyleIcons(XmlTableItem::ATTR_KEY, tr("key"), actionAttrKey);
	applyStyleIcons(XmlTableItem::ATTR_VAL, tr("value"), actionAttrVal);
	applyStyleIcons(XmlTableItem::TEXT, tr("text"), actionText);
	applyStyleIcons(XmlTableItem::CDATA, tr("cdata"), actionCDATA);
	applyStyleIcons(XmlTableItem::COMMENT, tr("comment"), actionComment);
	applyStyleIcons(XmlTableItem::EMPTY, tr("empty"), actionEmpty);
}


void Tx::applyStyleIcons(int type, const QString &label, QAction *action) {
	QPixmap icon(20,20);
	icon.fill( XmlTable::style(type)->bgColor() );
	QPainter p(&icon);
	QFont f = XmlTable::style(type)->font();
	p.setFont( f );
	p.setPen( QPen( XmlTable::style(type)->fgColor(), 1) );
	p.drawRect(0,0,19,19);
	p.drawText(0, 0, 20, 20, Qt::AlignLeft | Qt::AlignVCenter, label, -1, 0);
	action->setIconSet(icon);
}


void Tx::createFormatMenu() {
	setUsesTextLabel(TRUE);
	Q3ToolBar *tb = new Q3ToolBar( this );
	tb->setOrientation(Qt::Horizontal);
	tb->setLabel( tr("Format") );

	Q3PopupMenu * menu = new Q3PopupMenu( this );
	menuBar()->insertItem( tr("&Format"), menu );

	agrp = new QActionGroup(this);
	agrp->setExclusive(TRUE);
	connect(agrp, SIGNAL( selected(QAction*) ), this, SLOT( cellFormatSelected(QAction*) ) );
	ignoreToggle = FALSE;

	actionElement = new QAction ( tr("&Element"), agrp );
	actionElement->setShortcut(Qt::CTRL+Qt::Key_1);
	actionElement->setText( tr("<element") );
	actionElement->setToggleAction(TRUE);
	actionElement->setWhatsThis( tr("Sets the format of selected cell(s) to the ELEMENT type") );
	actionAttrKey = new QAction ( tr("Attribute &Key"), agrp );
	actionAttrKey->setShortcut(Qt::CTRL+Qt::Key_2);
	actionAttrKey->setText( tr("key=") );
	actionAttrKey->setToggleAction(TRUE);
	actionAttrKey->setWhatsThis( tr("Sets the format of selected cell(s) to the ATTRIBUTE-KEY type") );
	actionAttrVal = new QAction ( tr("Attribute &Value"), agrp );
	actionAttrVal->setShortcut(Qt::CTRL+Qt::Key_3);
	actionAttrVal->setText( tr("\"value\">") );
	actionAttrVal->setToggleAction(TRUE);
	actionAttrVal->setWhatsThis( tr("Sets the format of selected cell(s) to the ATTRIBUTE-VALUE type") );
	actionText = new QAction ( tr("&Text"), agrp );
	actionText->setShortcut(Qt::CTRL+Qt::Key_4);
	actionText->setText( tr("text") );
	actionText->setToggleAction(TRUE);
	actionText->setWhatsThis( tr("Sets the format of selected cell(s) to the TEXT type") );
	actionCDATA = new QAction ( tr("C&DATA"), agrp );
	actionCDATA->setShortcut(Qt::CTRL+Qt::Key_5);
	actionCDATA->setText( tr("<![CDATA[...]]>") );
	actionCDATA->setToggleAction(TRUE);
	actionCDATA->setWhatsThis( tr("Sets the format of selected cell(s) to the CDATA type") );
	actionComment = new QAction ( tr("C&omment"), agrp );
	actionComment->setShortcut(Qt::CTRL+Qt::Key_6);
	actionComment->setText( tr("<!--comment-->") );
	actionComment->setToggleAction(TRUE);
	actionComment->setWhatsThis( tr("Sets the format of selected cell(s) to the COMMENT type") );
	actionEmpty = new QAction ( tr("E&mpty"), agrp );
	actionEmpty->setShortcut(Qt::CTRL+Qt::Key_0);
	actionEmpty->setText( tr("______") );
	actionEmpty->setToggleAction(TRUE);
	actionEmpty->setWhatsThis( tr("Sets the format of selected cell(s) to the EMPTY type") );
	agrp->addTo(tb);
	agrp->addTo(menu);
	applyStyleIcons();

	menu->insertSeparator();
	menu->insertItem(QIcon(":images/adjustcellsizes.png"), tr("&Adjust Cell Sizes"), xtable, SLOT(adjustAll()), Qt::CTRL+Qt::Key_Return );
	menu->insertSeparator();
	menu->insertItem(QIcon(":images/shiftleft.png"), tr("Shift &Left"), xtable, SLOT(shiftLeft()), Qt::CTRL+Qt::Key_Left );
	menu->insertItem(QIcon(":images/shiftright.png"), tr("Shift &Right"), xtable, SLOT(shiftRight()), Qt::CTRL+Qt::Key_Right );
	menu->insertSeparator();
	menu->insertItem(QIcon(":images/insertrow.png"), tr("Insert Row"), this, SLOT(insertRow()), Qt::CTRL+Qt::Key_Equal );
	menu->insertItem(QIcon(":images/removerow.png"), tr("Remove Row"), this, SLOT(removeRow()), Qt::CTRL+Qt::Key_Minus );
	menu->insertSeparator();
	menu->insertItem(QIcon(":images/expand.png"), tr("Expand &Document"), xtable, SLOT(expandDoc()), Qt::CTRL+Qt::Key_D );
	menu->insertItem(QIcon(":images/trim.png"), tr("&Trim Document"), xtable, SLOT(trimDoc()), Qt::CTRL+Qt::Key_T );
}


void Tx::cellFormatSelected(QAction* act) {
	if (ignoreToggle) return;
	XmlTableItem::NodeType newType;
	if (act == actionElement) newType = XmlTableItem::ELEMENT;
	else if (act == actionAttrKey) newType = XmlTableItem::ATTR_KEY;
	else if (act == actionAttrVal) newType = XmlTableItem::ATTR_VAL;
	else if (act == actionText) newType = XmlTableItem::TEXT;
	else if (act == actionCDATA) newType = XmlTableItem::CDATA;
	else if (act == actionComment) newType = XmlTableItem::COMMENT;
	else if (act == actionEmpty) newType = XmlTableItem::EMPTY;
	else return;

	int minX = xtable->numCols(), minY = xtable->numRows(), maxX = 0, maxY = 0;
	int numSel = xtable->numSelections();
	for (int k = 0; k < numSel; k++) {
		Q3TableSelection sel = xtable->selection(k);
		if (sel.isEmpty()) continue;
		for (int i = sel.topRow(); i <= sel.bottomRow(); i++) {
			for (int j = sel.leftCol(); j <= sel.rightCol(); j++) {
				//minX = minX <? j;
				minX = (minX < j) ? minX : j;
				//minY = minY <? i;
				minY = (minY < i) ? minY : i;
				//maxX = maxX >? j;
				maxX = (maxX > j) ? maxX : j;
				//maxY = maxY >? i;
				maxY = (maxY > i) ? maxY : i;
			}
		}
	}
	xtable->addUndoRecord(minY, minX, maxY, maxX);
	XmlTableItem* currItem;
	for (int k = 0; k < numSel; k++) {
		Q3TableSelection sel = xtable->selection(k);
		if (sel.isEmpty()) continue;
		for (int i = sel.topRow(); i <= sel.bottomRow(); i++) {
			for (int j = sel.leftCol(); j <= sel.rightCol(); j++) {
				currItem = xtable->xmlItem(i, j);
				if (currItem == NULL) {
					currItem = new XmlTableItem(newType, xtable, Q3TableItem::OnTyping, "");
					xtable->setItem(i, j, currItem);
				}
				else {
					currItem->setItemType(newType);
					xtable->updateCell(i, j);
					if (currItem->isSpanCell()) j = sel.rightCol() + 1;
				}
			}
		}
	}
	xtable->setModified(TRUE);
}



void Tx::createHelpMenu() {
	menuBar()->insertSeparator();
	Q3PopupMenu * help = new Q3PopupMenu( this );
	menuBar()->insertItem( tr("&Help"), help );
	help->insertItem(QIcon(":images/info.png"), tr("What's &This"), this, SLOT(whatsThis()), Qt::SHIFT+Qt::Key_F1 );
	help->insertItem(QIcon(":images/help.png"), tr("Online Help..."), this, SLOT(onlineHelp()) );
	help->insertSeparator();
	help->insertItem(QIcon(":images/tXsmall.png"), tr("&About tx"), this, SLOT(about()), Qt::Key_F1 );
	help->insertItem(QIcon(":images/about.png"), tr("About &Qt"), this, SLOT(aboutQt()) );
}


void Tx::createStatusBar() {
	rowLabel = new QLabel( tr("Row: 999"), this );
	rowLabel->setAlignment(Qt::AlignHCenter);
	rowLabel->setMinimumSize(rowLabel->sizeHint());
	rowLabel->setText( tr("Row: 1") );

	colLabel = new QLabel( tr("Col: 999"), this );
	colLabel->setAlignment(Qt::AlignHCenter);
	colLabel->setMinimumSize(colLabel->sizeHint());
	colLabel->setText( tr("Col: 1") );

	typeLabel = new QLabel(" 9999999 ", this);
	typeLabel->setFont( XmlTable::style(XmlTableItem::ELEMENT)->font() );
	typeLabel->setMinimumSize(typeLabel->sizeHint());
	typeLabel->setAlignment(Qt::AlignHCenter);
	typeLabel->clear();

	statusLabel = new QLabel(this);

	modLabel = new QLabel(tr("MOD"), this);
	modLabel->setAlignment(Qt::AlignHCenter);
	modLabel->setMinimumSize(modLabel->sizeHint());
	modLabel->clear();

	statusBar()->addWidget(rowLabel, 0, TRUE);
	statusBar()->addWidget(colLabel, 0, TRUE);
	statusBar()->addWidget(typeLabel, 0, TRUE);
	statusBar()->addWidget(statusLabel, 1, TRUE);
	statusBar()->addWidget(modLabel, 0, TRUE);

	connect( xtable, SIGNAL( currentChanged(int, int) ), this, SLOT( updateCellIndicators(int, int) ) );
	connect( xtable, SIGNAL( modificationChanged(bool) ), this, SLOT( tableModified(bool) ) );
	connect(
		statusBar(), SIGNAL( messageChanged(const QString &) ),
		statusLabel, SLOT( setText(const QString &) )
		);
}


void Tx::updateCellIndicators(int row, int col) {
	rowLabel->setText( tr("Row: %1").arg(row+1) );
	colLabel->setText( tr("Col: %2").arg(col+1) );
	cellEdit->setText( xtable->text(row, col) );
	XmlTableItem::NodeType currType = xtable->itemType(row, col);
	typeLabel->setText( XmlTable::style(currType)->name() );
	typeLabel->setFont( XmlTable::style(currType)->font() );
	typeLabel->setPaletteForegroundColor( XmlTable::style(currType)->fgColor() );
	typeLabel->setPaletteBackgroundColor( XmlTable::style(currType)->bgColor() );

	ignoreToggle = TRUE;
	switch (currType) {
		case XmlTableItem::ELEMENT: 
			currentElementText = xtable->text(row, col);
			if(QApplication::mouseButtons() == Qt::RightButton)
			{
				helpText->setSource(QString("%1.html").arg(currentElementText));
			}
			actionElement->setOn(TRUE);
			break;
		case XmlTableItem::ATTR_KEY: 
			if((QApplication::mouseButtons() == Qt::RightButton) && !currentElementText.isNull())
			{
				helpText->setSource(QString("%1.html#%2").arg(currentElementText).arg(xtable->text(row, col)));
			}
			actionAttrKey->setOn(TRUE); 
			break;
		case XmlTableItem::ATTR_VAL: 
			actionAttrVal->setOn(TRUE); 
			break;
		case XmlTableItem::TEXT: 
			actionText->setOn(TRUE); 
			break;
		case XmlTableItem::CDATA: 
			actionCDATA->setOn(TRUE); 
			break;
		case XmlTableItem::COMMENT: 
			actionComment->setOn(TRUE); 
			break;
		case XmlTableItem::EMPTY: 
			actionEmpty->setOn(TRUE); 
			break;
		default: break;
	}



	ignoreToggle = FALSE;
	xtable->updateAll();
}


void Tx::tableModified(bool ismod) {
	if (ismod) modLabel->setText(tr("MOD"));
	else modLabel->clear();
}


void Tx::setMatchCase(bool mc) {
	matchCase = mc;
}


void Tx::newDoc() {
	Tx *ed = new Tx;
	ed->setCaption("tx");
	ed->show();
}


void Tx::choose() {
	QString fn;
	QSettings settings;
	settings.setPath(TX_SETTINGS_PATH, "tx", QSettings::UserScope);
	settings.beginGroup("/tx");
	QString lastDir = settings.readEntry( "/dir", QString::null );
	settings.endGroup();
	fn = QFileDialog::getOpenFileName( this, tr("Open File"), lastDir);
	if ( !fn.isNull() ) {
		QDir dir;
		settings.beginGroup("/tx");
		settings.writeEntry( "/dir", dir.filePath(fn) );
		settings.endGroup();
	}

	if ( !fn.isEmpty() ) load( fn );
	else message( tr("Loading aborted") );
}


void Tx::setActiveWindow() {
	QWidget::setActiveWindow();
}


void Tx::load( const QString &fn, bool isReload ) {
	currentElementText = QString();
	if (!isReload) {
		QWidgetList openAppList = QApplication::topLevelWidgets();
		QWidget* w;
		for (int i = 0; i < openAppList.size(); ++i) 
		{
			w = openAppList.at(i);
			w->raise();
			w->setActiveWindow();
		}
	}

	if (! this->filename.isEmpty()) {
		closeEvent(NULL);
	}

	clearMessages();
	QFile f( fn );
	if ( !f.open( QIODevice::ReadOnly ) ) {
		message( tr("Unable to open file: %1").arg(fn), "", TRUE, TRUE, QMessageBox::Critical );
		return;
	}

	setCaption(fn + " - tx");
	this->filename = fn;
	setName(fn);
	if ( xtable->setContent(&f) ) {
		message( tr("Loaded document %1").arg(fn) );
	}
	updateRecentDocList(fn);
	updateFileMenu();

	QSettings settings;
	settings.setPath(TX_SETTINGS_PATH, "tx", QSettings::UserScope);
	settings.beginGroup("/tx");

	QFileInfo fi(f);
	QString path = fi.absoluteDir().absolutePath();								//file-directory + DTD-name
	path = path + "/" + xtable->dtdName();

	QStringList qsl;
	if(QDir(path).exists())
		qsl << path;
	path = QDir::currentPath() + "/" + xtable->dtdName();					//working-directory + DTD-name
	if(QDir(path).exists())
		qsl << path;
	path = settings.readEntry( "/helpbrowserpath", QString() );		//global-directory + DTD-name
	if(!path.isNull())
	{
		path += "/" + xtable->dtdName();
		if(QDir(path).exists())
			qsl << path;
	}
	helpText->setSearchPaths(qsl);
}


void Tx::reload() {
	load( this->filename, TRUE );
}



void Tx::save() {
	clearMessages();
	//xtable->setCurrentCell(0, 0);
	if ( filename.isEmpty() ) {
		saveAs();
		return;
	}

	QFile f( filename );
	if ( !f.open(QIODevice::WriteOnly) ) {
		message( tr("Could not write to file: %1").arg(filename), "", TRUE, TRUE, QMessageBox::Critical );
		return;
	}

	QTextStream t( &f );
	if (XmlTable::outputEncoding == "UTF-16") {
		t.setEncoding(QTextStream::Unicode);
	}
	else if (XmlTable::outputEncoding == "UTF-8") {
		t.setEncoding(QTextStream::UnicodeUTF8);
	}
	else {
		QTextCodec *codec = QTextCodec::codecForName(XmlTable::outputEncoding);
		if (codec != NULL) {
			t.setEncoding(QTextStream::Locale);
			t.setCodec(codec);
		}
		else {
			XmlTable::outputEncoding = "UTF-16";
			t.setEncoding(QTextStream::Unicode);
		}
	}
	xtable->save(t, dtdEdit->text());
	f.close();

	setCaption( filename + " - tx");
	message( tr( "File %1 saved" ).arg( filename ) );
}


void Tx::saveAs() {
	QString fn = QFileDialog::getSaveFileName( QString::null, QString::null, this );
	if ( fn.isEmpty() ) {
		message( tr("Saving aborted") );
		return;
	}
	filename = fn;
	save();
}


void Tx::exportFile() {
	clearMessages();
	QString fn = QFileDialog::getSaveFileName( QString::null, "HTML (*.html *.htm)", this );
	if ( fn.isEmpty() ) {
		message( tr("Export aborted") );
		return;
	}
	if (!( fn.endsWith(".html", FALSE) || fn.endsWith(".htm", FALSE) )) fn += ".html";
	QFile f( fn );
	if ( !f.open(QIODevice::WriteOnly) ) {
		message( tr("Could not write to file: %1").arg(filename), "", TRUE, TRUE, QMessageBox::Critical );
		return;
	}

	QTextStream t( &f );
	t << xtable->html_css(0, 0, xtable->maxRows(), xtable->maxCols());
	f.close();

	message( tr( "File %1 saved" ).arg( fn ) );
}


void Tx::print() {
	printRichText(xtable->html2( 0, 0, xtable->maxRows(), xtable->maxCols()));
}


void Tx::printRichText(const QString &str) {
	if (!printer->setup(this)) return;
	Q3Painter painter(printer);
	int pageHeight = painter.window().height() - 2 * 48;
	Q3SimpleRichText richText(str, bodyFont, "", 0, 0, pageHeight);
	richText.setWidth(&painter, painter.window().width());
	int numPages = (int)ceil( (double)richText.height() / pageHeight );
	int index;
	for (int i = 0; i < (int)printer->numCopies(); i++) {
		for (int j = 0; j < numPages; j++) {
			if (i > 0 || j > 0) printer->newPage();
			if (printer->pageOrder() == QPrinter::LastPageFirst) {
				index = numPages - j - 1;
			}
			else index = j;
			printPage(&painter, richText, pageHeight, index);
		}
	}
}


void Tx::printPage(Q3Painter *painter, const Q3SimpleRichText &richText, int pageHeight, int index) {
	QRect rect(0, index * pageHeight + 48, richText.width(), pageHeight);
	painter->translate(0, -rect.y());
	richText.draw(painter, 0, 48, rect, colorGroup());
	painter->resetMatrix();
	painter->setFont(footerFont);
	painter->drawText( painter->window(), Qt::AlignHCenter | Qt::AlignBottom, QString::number(index+1) );
}


void Tx::closeEvent( QCloseEvent* ce ) {
	if ( !xtable->isModified() ) {
		saveSettings();
		if (ce != NULL) ce->accept();
	}
	else {
		int ask = QMessageBox::information( this, "tx",
			tr("Do you want to save the changes to the document?"),
			tr("Yes"), tr("No"), tr("Cancel"), 0, 1
			);
		switch(ask) {
			case 0:
				save();
				if (ce != NULL) ce->accept();
				break;
			case 1:
				if (ce != NULL) ce->accept();
				break;
			case 2:
			default:
				if (ce != NULL) ce->ignore();
				break;
		}
	}
}


void Tx::cut() {
	copy();
	del();
}


void Tx::copy() {
	if (xtable->hasFocus()) xtable->copy(FALSE);
	else if (cellEdit->hasFocus()) cellEdit->copy();
}


void Tx::copyAsText() {
	if (xtable->hasFocus()) xtable->copy(TRUE);
	else if (cellEdit->hasFocus()) cellEdit->copy();
}


void Tx::paste() {
	if (xtable->hasFocus()) xtable->paste();
	else if (cellEdit->hasFocus()) cellEdit->paste();
}


void Tx::del() {
	if (xtable->hasFocus()) xtable->del();
	else if (cellEdit->hasFocus()) cellEdit->del();
}


void Tx::selectAll() {
	if (xtable->hasFocus()) xtable->selectCells( 0, 0, xtable->maxRows() /*xtable->numRows()*/, xtable->maxCols() /*xtable->numCols()*/ );
	else if (cellEdit->hasFocus()) cellEdit->selectAll(TRUE);
}


void Tx::find() {
	findToolBar->show();
	findEdit->setFocus();
	findEdit->selectAll();
}


void Tx::docType() {
	bottomTabs->showPage(dtdEdit);
	dtdEdit->setFocus();
}

void Tx::setTabEditCell() {
	bottomTabs->showPage(cellEdit);
}


void Tx::findNext() {
	QString val = findEdit->text();
	if (val.isEmpty()) return;
	xtable->clearSelection(TRUE);
	int r = xtable->currentRow(), c = xtable->currentColumn();
	if ( (r >= xtable->maxRows()) && (c >= xtable->maxCols()) ) {
		r = 0; c = 0;
	}
	Q3TableItem* currItem;
	for (int i = 0; r <= xtable->maxRows(); r++, i++) {
		if (i > 0) c = 0;
		for (; c <= xtable->maxCols(); c++) {
			currItem = xtable->item(r, c);
			if (currItem == NULL) continue;
			if (currItem->row() != r) continue;
			if (currItem->col() != c) continue;
			if (xtable->text(r, c).find(val, 0, matchCase) < 0) continue;
			xtable->setCurrentCell(r, c);
			if (! cellEdit->find(val, matchCase, FALSE, TRUE) ) continue;
			cellEdit->setFocus();
			return;
		}
	}
	xtable->setCurrentCell(0, -1);
	xtable->clearSelection(TRUE);
	message( tr("End of document reached") );
}


void Tx::findPrevious() {
	QString val = findEdit->text();
	if (val.isEmpty()) return;
	xtable->clearSelection(TRUE);
	int r = xtable->currentRow(), c = xtable->currentColumn();
	if ( (r <= 0) && (c <= 0) ) {
		r = xtable->maxRows();
		c = xtable->maxCols();
	}
	Q3TableItem* currItem;
	for (int i = 0; r >= 0; r--, i++) {
		if (i > 0) c = xtable->maxCols();
		for (; c >= 0; c--) {
			currItem = xtable->item(r, c);
			if (currItem == NULL) continue;
			if (currItem->row() != r) continue;
			if (currItem->col() != c) continue;
			if (xtable->text(r, c).find(val, 0, matchCase) < 0) continue;
			xtable->setCurrentCell(r, c);
			if (! cellEdit->find(val, matchCase, FALSE, FALSE) ) continue;
			cellEdit->setFocus();
			return;
		}
	}
	xtable->setCurrentCell(0, -1);
	xtable->clearSelection(TRUE);
	message( tr("End of document reached") );
}


void Tx::replace() {
	find();
	replaceToolBar->show();
}


void Tx::replaceNext() {
	int r = xtable->currentRow(), c = xtable->currentColumn();
	QString currVal = xtable->text(r, c);
	if (cellEdit->hasSelectedText()) {
		cellEdit->insert(replaceEdit->text());
		xtable->updateCell(r, c);
	}
	findNext();
	xtable->setModified(TRUE);
}


void Tx::replaceAll() {
	Q3TableItem *currItem;
	xtable->clearSelection(TRUE);
	QString val = findEdit->text();
	int numCellsReplaced = 0;
	xtable->addUndoRecord(0, 0, xtable->maxRows(), xtable->maxCols());
	for (int r = 0; r <= xtable->maxRows(); r++) {
		for (int c = 0; c <= xtable->maxCols(); c++) {
			currItem = xtable->item(r, c);
			if (currItem == NULL) continue;
			if (currItem->row() != r) continue;
			if (currItem->col() != c) continue;
			QString currVal = xtable->text(r, c);
			if (currVal.find(val, 0, matchCase) < 0) continue;
			currVal.replace( findEdit->text(), replaceEdit->text(), matchCase );
			xtable->setText(r, c, currVal);
			xtable->updateCell(r, c);
			numCellsReplaced++;
		}
	}
	if (numCellsReplaced == 0) return;
	xtable->setModified(TRUE);
	QMessageBox::information( this, tr("Replace All"),
		tr("Text in %1 cell(s) replaced").arg(numCellsReplaced) 
		);
}


void Tx::preferences() {
	pref->show();
	pref->raise();
}


void Tx::insertRow() {
	int r = xtable->currentRow(), c = xtable->currentColumn();
	xtable->insertRows( r, 1 );
	xtable->setCurrentCell(r, c);
	xtable->resetUndo();
	xtable->resetRedo();
	for (; r < xtable->numRows(); r++) xtable->adjustRow(r);
}


void Tx::removeRow() {
	int r = xtable->currentRow(), c = xtable->currentColumn();
	xtable->removeRow( xtable->currentRow() );
	xtable->setCurrentCell(r, c);
	xtable->resetUndo();
	xtable->resetRedo();
}


void Tx::onlineHelp() {
	openWebLink("http://www.gravitybind.com/help.html");
}


void Tx::about() {
	(new AboutTxDialog(this))->show();
	/*
	QMessageBox::about( this, tr("About")+" tX", tr(
	"<h1><b>tX</b>   <small>version 1.2</small></h1>"
	"<nobr><h4><b>T</b>abular <b>X</b>ML editor.</h4></nobr>"
	"<br>See <tt>http://www.gravitybind.com</tt> for more information."
	"<br>"
	"<br><nobr>Created by: Jinsoo Kang (<tt>jsk@gravitybind.com</tt>)</nobr>"
	"<br>"
	"<br><nobr>Developers: Timm Reichert (<tt>tr@reichert-gmbh.com</tt>)</nobr>"
	"<br>"
	"<br><nobr>Web Design / Branding: Trillamar Technology Marketing(<tt>www.trillamar.com</tt>)</nobr>"
	"<br>"
	"<br><small>Copyright (C) 2005-2006 by Jinsoo Kang</small>"
	));
	*/
}


void Tx::aboutQt() {
	QMessageBox::aboutQt( this, tr("About Qt") );
}

