/***************************************************************************
*   Copyright (C) 2005-2006 by Jinsoo Kang                                *
*   jsk@gravitybind.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef TX_H
#define TX_H

#include <QtGui>
#include <QtCore>

#include "xmltable.h"
#include "prefdialog.h"
#include "messagelist.h"


class HelpBrowser;

class Tx: public Q3MainWindow {
	Q_OBJECT

public:
	Tx();
	~Tx();

	void setActiveWindow();

protected:
	void closeEvent( QCloseEvent* );


	public slots:

		void updateCellIndicators(int row, int col);
		void tableModified(bool ismod);
		void load( const QString &fileName, bool isReload = FALSE );
		void message(
			const QString &msg, const QString &loc = "", 
			bool printStatusBar = TRUE, bool printLog = FALSE,
			QMessageBox::Icon icn = QMessageBox::Information
			);
		void clearMessages();


		private slots:
			void updateCellText();
			void newDoc();
			void choose();
			void save();
			void saveAs();
			void reload();
			void openAsText();
			void print();

			void cellFormatSelected(QAction* act);

			void exportFile();
			void cut();
			void copy();
			void copyAsText();

			void paste();
			void del();
			void selectAll();
			void find();
			void findNext();
			void findPrevious();
			void replace();
			void replaceNext();
			void replaceAll();
			void docType();
			void setTabEditCell();
			void setMatchCase(bool mc);
			void hideReplaceToolBar(bool vis);

			void preferences();

			void insertRow();
			void removeRow();

			void onlineHelp();
			void about();
			void aboutQt();

			void applyStyleIcons();
			void applyStyleIcons(int type, const QString &label, QAction *action);
			void openMessageTarget(Q3ListViewItem *item);

			void openRecentFile(int id);

			void updateRecentDocMenu();
			void updateRecentDocList(const QString& newitem = QString::null );

			void updateUndoMenu(int size);
			void updateRedoMenu(int size);
			void updateFileMenu();

private:
	void applySettings();
	void saveSettings();

	void createFileMenu();
	void createEditMenu();
	void createFormatMenu();
	void createHelpMenu();


	void createStatusBar();
	void printRichText(const QString &str);
	void printPage(Q3Painter *painter, const Q3SimpleRichText &richText, int pageHeight, int index);


	QActionGroup *agrp;
	QAction *actionElement,
		*actionAttrKey,
		*actionAttrVal,
		*actionText,
		*actionCDATA,
		*actionComment,
		*actionEmpty;


	QPrinter *printer;
	QString filename;

	XmlTable *xtable;
	Q3TextEdit *cellEdit;
	Q3TextEdit *dtdEdit;
	QWidget *errorTable;
	HelpBrowser *helpText;

	PrefDialog *pref;

	//find / replace
	Q3ToolBar *findToolBar, *replaceToolBar;
	QLineEdit *findEdit, *replaceEdit;
	bool matchCase;

	//status bar
	QLabel *rowLabel;
	QLabel *colLabel;
	QLabel *typeLabel;
	QLabel *statusLabel;
	QLabel *modLabel;

	QTabWidget *bottomTabs;

	bool ignoreToggle;

	QFont bodyFont, footerFont;

	QString currentElementText;

	static Q3IntDict<QPixmap> messageIcons;

	Q3PopupMenu *recentMenu;
	static QStringList recentDocList;

	Q3PopupMenu *editMenu, *fileMenu;
	int undoMenuId, redoMenuId, openAsTextMenuId, reloadMenuId;



public:
	static QString externalEditor;
	static bool externalEditorActive;
	static int numRecentDocs;

};


#endif
