/**************************************************************************
*   Copyright (C) 2005-2006 by Jinsoo Kang                                *
*   jsk@gravitybind.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
**************************************************************************/



#include "tx_util.h"


void openWebLink(const QString& addr) {
#if defined(Q_WS_WIN)
	QString launch("C:\\Program Files\\Internet Explorer\\iexplore.exe");
#endif
#if defined(Q_WS_MAC)
	QString launch("/usr/bin/open"); 
#endif
#if defined(Q_WS_X11)
	QString launch(getenv("DESKTOP_LAUNCH")); 
#endif
#if defined(Q_WS_QWS)
	QString launch(getenv("DESKTOP_LAUNCH")); 
#endif
	Q3Process proc(launch);
	proc.addArgument(addr);
	proc.launch(QString(""));
}


bool isLetter(const QChar &c) {
	if (isBaseChar(c) || isIdeographic(c)) return TRUE;
	return FALSE;
}


bool isDigit(const QChar &c) {
	if (
		inRange(c, 0x0030,0x0039) || inRange(c, 0x0660,0x0669) || inRange(c, 0x06F0,0x06F9) 
		|| inRange(c, 0x0966,0x096F) || inRange(c, 0x09E6,0x09EF) || inRange(c, 0x0A66,0x0A6F) 
		|| inRange(c, 0x0AE6,0x0AEF) || inRange(c, 0x0B66,0x0B6F) || inRange(c, 0x0BE7,0x0BEF) 
		|| inRange(c, 0x0C66,0x0C6F) || inRange(c, 0x0CE6,0x0CEF) || inRange(c, 0x0D66,0x0D6F) 
		|| inRange(c, 0x0E50,0x0E59) || inRange(c, 0x0ED0,0x0ED9) || inRange(c, 0x0F20,0x0F29)
		) return TRUE;
	return FALSE;
}


bool isCombiningChar(const QChar &c) {
	if (
		inRange(c, 0x0300,0x0345) || inRange(c, 0x0360,0x0361) || inRange(c, 0x0483,0x0486) 
		|| inRange(c, 0x0591,0x05A1) || inRange(c, 0x05A3,0x05B9) || inRange(c, 0x05BB,0x05BD) 
		|| inRange(c, 0x05BF)        || inRange(c, 0x05C1,0x05C2) || inRange(c, 0x05C4) 
		|| inRange(c, 0x064B,0x0652) || inRange(c, 0x0670)        || inRange(c, 0x06D6,0x06DC) 
		|| inRange(c, 0x06DD,0x06DF) || inRange(c, 0x06E0,0x06E4) || inRange(c, 0x06E7,0x06E8) 
		|| inRange(c, 0x06EA,0x06ED) || inRange(c, 0x0901,0x0903) || inRange(c, 0x093C) 
		|| inRange(c, 0x093E,0x094C) || inRange(c, 0x094D)        || inRange(c, 0x0951,0x0954) 
		|| inRange(c, 0x0962,0x0963) || inRange(c, 0x0981,0x0983) || inRange(c, 0x09BC) 
		|| inRange(c, 0x09BE)        || inRange(c, 0x09BF)        || inRange(c, 0x09C0,0x09C4) 
		|| inRange(c, 0x09C7,0x09C8) || inRange(c, 0x09CB,0x09CD) || inRange(c, 0x09D7) 
		|| inRange(c, 0x09E2,0x09E3) || inRange(c, 0x0A02)        || inRange(c, 0x0A3C) 
		|| inRange(c, 0x0A3E)        || inRange(c, 0x0A3F)        || inRange(c, 0x0A40,0x0A42) 
		|| inRange(c, 0x0A47,0x0A48) || inRange(c, 0x0A4B,0x0A4D) || inRange(c, 0x0A70,0x0A71) 
		|| inRange(c, 0x0A81,0x0A83) || inRange(c, 0x0ABC)        || inRange(c, 0x0ABE,0x0AC5) 
		|| inRange(c, 0x0AC7,0x0AC9) || inRange(c, 0x0ACB,0x0ACD) || inRange(c, 0x0B01,0x0B03) 
		|| inRange(c, 0x0B3C)        || inRange(c, 0x0B3E,0x0B43) || inRange(c, 0x0B47,0x0B48) 
		|| inRange(c, 0x0B4B,0x0B4D) || inRange(c, 0x0B56,0x0B57) || inRange(c, 0x0B82,0x0B83) 
		|| inRange(c, 0x0BBE,0x0BC2) || inRange(c, 0x0BC6,0x0BC8) || inRange(c, 0x0BCA,0x0BCD) 
		|| inRange(c, 0x0BD7)        || inRange(c, 0x0C01,0x0C03) || inRange(c, 0x0C3E,0x0C44) 
		|| inRange(c, 0x0C46,0x0C48) || inRange(c, 0x0C4A,0x0C4D) || inRange(c, 0x0C55,0x0C56) 
		|| inRange(c, 0x0C82,0x0C83) || inRange(c, 0x0CBE,0x0CC4) || inRange(c, 0x0CC6,0x0CC8) 
		|| inRange(c, 0x0CCA,0x0CCD) || inRange(c, 0x0CD5,0x0CD6) || inRange(c, 0x0D02,0x0D03) 
		|| inRange(c, 0x0D3E,0x0D43) || inRange(c, 0x0D46,0x0D48) || inRange(c, 0x0D4A,0x0D4D) 
		|| inRange(c, 0x0D57)        || inRange(c, 0x0E31)        || inRange(c, 0x0E34,0x0E3A) 
		|| inRange(c, 0x0E47,0x0E4E) || inRange(c, 0x0EB1)        || inRange(c, 0x0EB4,0x0EB9) 
		|| inRange(c, 0x0EBB,0x0EBC) || inRange(c, 0x0EC8,0x0ECD) || inRange(c, 0x0F18,0x0F19) 
		|| inRange(c, 0x0F35)        || inRange(c, 0x0F37)        || inRange(c, 0x0F39) 
		|| inRange(c, 0x0F3E)        || inRange(c, 0x0F3F)        || inRange(c, 0x0F71,0x0F84) 
		|| inRange(c, 0x0F86,0x0F8B) || inRange(c, 0x0F90,0x0F95) || inRange(c, 0x0F97) 
		|| inRange(c, 0x0F99,0x0FAD) || inRange(c, 0x0FB1,0x0FB7) || inRange(c, 0x0FB9) 
		|| inRange(c, 0x20D0,0x20DC) || inRange(c, 0x20E1)        || inRange(c, 0x302A,0x302F) 
		|| inRange(c, 0x3099)        || inRange(c, 0x309A)
		) return TRUE;
	return FALSE;
}


bool isExtender(const QChar &c) {
	if (
		inRange(c, 0x00B7)        || inRange(c, 0x02D0)        || inRange(c, 0x02D1) 
		|| inRange(c, 0x0387)        || inRange(c, 0x0640)        || inRange(c, 0x0E46) 
		|| inRange(c, 0x0EC6)        || inRange(c, 0x3005)        || inRange(c, 0x3031,0x3035) 
		|| inRange(c, 0x309D,0x309E) || inRange(c, 0x30FC,0x30FE)
		) return TRUE;
	return FALSE;
}


bool isBaseChar(const QChar &c) {
	if (
		inRange(c, 0x0041,0x005A) || inRange(c, 0x0061,0x007A) || inRange(c, 0x00C0,0x00D6) 
		|| inRange(c, 0x00D8,0x00F6) || inRange(c, 0x00F8,0x00FF) || inRange(c, 0x0100,0x0131) 
		|| inRange(c, 0x0134,0x013E) || inRange(c, 0x0141,0x0148) || inRange(c, 0x014A,0x017E) 
		|| inRange(c, 0x0180,0x01C3) || inRange(c, 0x01CD,0x01F0) || inRange(c, 0x01F4,0x01F5) 
		|| inRange(c, 0x01FA,0x0217) || inRange(c, 0x0250,0x02A8) || inRange(c, 0x02BB,0x02C1) 
		|| inRange(c, 0x0386)        || inRange(c, 0x0388,0x038A) || inRange(c, 0x038C) 
		|| inRange(c, 0x038E,0x03A1) || inRange(c, 0x03A3,0x03CE) || inRange(c, 0x03D0,0x03D6) 
		|| inRange(c, 0x03DA)        || inRange(c, 0x03DC)        || inRange(c, 0x03DE) 
		|| inRange(c, 0x03E0)        || inRange(c, 0x03E2,0x03F3) || inRange(c, 0x0401,0x040C) 
		|| inRange(c, 0x040E,0x044F) || inRange(c, 0x0451,0x045C) || inRange(c, 0x045E,0x0481) 
		|| inRange(c, 0x0490,0x04C4) || inRange(c, 0x04C7,0x04C8) || inRange(c, 0x04CB,0x04CC) 
		|| inRange(c, 0x04D0,0x04EB) || inRange(c, 0x04EE,0x04F5) || inRange(c, 0x04F8,0x04F9) 
		|| inRange(c, 0x0531,0x0556) || inRange(c, 0x0559)        || inRange(c, 0x0561,0x0586) 
		|| inRange(c, 0x05D0,0x05EA) || inRange(c, 0x05F0,0x05F2) || inRange(c, 0x0621,0x063A) 
		|| inRange(c, 0x0641,0x064A) || inRange(c, 0x0671,0x06B7) || inRange(c, 0x06BA,0x06BE) 
		|| inRange(c, 0x06C0,0x06CE) || inRange(c, 0x06D0,0x06D3) || inRange(c, 0x06D5) 
		|| inRange(c, 0x06E5,0x06E6) || inRange(c, 0x0905,0x0939) || inRange(c, 0x093D) 
		|| inRange(c, 0x0958,0x0961) || inRange(c, 0x0985,0x098C) || inRange(c, 0x098F,0x0990) 
		|| inRange(c, 0x0993,0x09A8) || inRange(c, 0x09AA,0x09B0) || inRange(c, 0x09B2) 
		|| inRange(c, 0x09B6,0x09B9) || inRange(c, 0x09DC,0x09DD) || inRange(c, 0x09DF,0x09E1) 
		|| inRange(c, 0x09F0,0x09F1) || inRange(c, 0x0A05,0x0A0A) || inRange(c, 0x0A0F,0x0A10) 
		|| inRange(c, 0x0A13,0x0A28) || inRange(c, 0x0A2A,0x0A30) || inRange(c, 0x0A32,0x0A33) 
		|| inRange(c, 0x0A35,0x0A36) || inRange(c, 0x0A38,0x0A39) || inRange(c, 0x0A59,0x0A5C) 
		|| inRange(c, 0x0A5E)        || inRange(c, 0x0A72,0x0A74) || inRange(c, 0x0A85,0x0A8B) 
		|| inRange(c, 0x0A8D)        || inRange(c, 0x0A8F,0x0A91) || inRange(c, 0x0A93,0x0AA8) 
		|| inRange(c, 0x0AAA,0x0AB0) || inRange(c, 0x0AB2,0x0AB3) || inRange(c, 0x0AB5,0x0AB9) 
		|| inRange(c, 0x0ABD)        || inRange(c, 0x0AE0)        || inRange(c, 0x0B05,0x0B0C) 
		|| inRange(c, 0x0B0F,0x0B10) || inRange(c, 0x0B13,0x0B28) || inRange(c, 0x0B2A,0x0B30) 
		|| inRange(c, 0x0B32,0x0B33) || inRange(c, 0x0B36,0x0B39) || inRange(c, 0x0B3D) 
		|| inRange(c, 0x0B5C,0x0B5D) || inRange(c, 0x0B5F,0x0B61) || inRange(c, 0x0B85,0x0B8A) 
		|| inRange(c, 0x0B8E,0x0B90) || inRange(c, 0x0B92,0x0B95) || inRange(c, 0x0B99,0x0B9A) 
		|| inRange(c, 0x0B9C)        || inRange(c, 0x0B9E,0x0B9F) || inRange(c, 0x0BA3,0x0BA4) 
		|| inRange(c, 0x0BA8,0x0BAA) || inRange(c, 0x0BAE,0x0BB5) || inRange(c, 0x0BB7,0x0BB9) 
		|| inRange(c, 0x0C05,0x0C0C) || inRange(c, 0x0C0E,0x0C10) || inRange(c, 0x0C12,0x0C28) 
		|| inRange(c, 0x0C2A,0x0C33) || inRange(c, 0x0C35,0x0C39) || inRange(c, 0x0C60,0x0C61) 
		|| inRange(c, 0x0C85,0x0C8C) || inRange(c, 0x0C8E,0x0C90) || inRange(c, 0x0C92,0x0CA8) 
		|| inRange(c, 0x0CAA,0x0CB3) || inRange(c, 0x0CB5,0x0CB9) || inRange(c, 0x0CDE) 
		|| inRange(c, 0x0CE0,0x0CE1) || inRange(c, 0x0D05,0x0D0C) || inRange(c, 0x0D0E,0x0D10) 
		|| inRange(c, 0x0D12,0x0D28) || inRange(c, 0x0D2A,0x0D39) || inRange(c, 0x0D60,0x0D61) 
		|| inRange(c, 0x0E01,0x0E2E) || inRange(c, 0x0E30)        || inRange(c, 0x0E32,0x0E33) 
		|| inRange(c, 0x0E40,0x0E45) || inRange(c, 0x0E81,0x0E82) || inRange(c, 0x0E84) 
		|| inRange(c, 0x0E87,0x0E88) || inRange(c, 0x0E8A)        || inRange(c, 0x0E8D) 
		|| inRange(c, 0x0E94,0x0E97) || inRange(c, 0x0E99,0x0E9F) || inRange(c, 0x0EA1,0x0EA3) 
		|| inRange(c, 0x0EA5)        || inRange(c, 0x0EA7)        || inRange(c, 0x0EAA,0x0EAB) 
		|| inRange(c, 0x0EAD,0x0EAE) || inRange(c, 0x0EB0)        || inRange(c, 0x0EB2,0x0EB3) 
		|| inRange(c, 0x0EBD)        || inRange(c, 0x0EC0,0x0EC4) || inRange(c, 0x0F40,0x0F47) 
		|| inRange(c, 0x0F49,0x0F69) || inRange(c, 0x10A0,0x10C5) || inRange(c, 0x10D0,0x10F6) 
		|| inRange(c, 0x1100)        || inRange(c, 0x1102,0x1103) || inRange(c, 0x1105,0x1107) 
		|| inRange(c, 0x1109)        || inRange(c, 0x110B,0x110C) || inRange(c, 0x110E,0x1112) 
		|| inRange(c, 0x113C)        || inRange(c, 0x113E)        || inRange(c, 0x1140)
		|| inRange(c, 0x114C)        || inRange(c, 0x114E)        || inRange(c, 0x1150)
		|| inRange(c, 0x1154,0x1155) || inRange(c, 0x1159)        || inRange(c, 0x115F,0x1161)
		|| inRange(c, 0x1163)        || inRange(c, 0x1165)        || inRange(c, 0x1167)
		|| inRange(c, 0x1169)        || inRange(c, 0x116D,0x116E) || inRange(c, 0x1172,0x1173)
		|| inRange(c, 0x1175)        || inRange(c, 0x119E)        || inRange(c, 0x11A8)
		|| inRange(c, 0x11AB)        || inRange(c, 0x11AE,0x11AF) || inRange(c, 0x11B7,0x11B8)
		|| inRange(c, 0x11BA)        || inRange(c, 0x11BC,0x11C2) || inRange(c, 0x11EB)
		|| inRange(c, 0x11F0)        || inRange(c, 0x11F9)        || inRange(c, 0x1E00,0x1E9B)
		|| inRange(c, 0x1EA0,0x1EF9) || inRange(c, 0x1F00,0x1F15) || inRange(c, 0x1F18,0x1F1D)
		|| inRange(c, 0x1F20,0x1F45) || inRange(c, 0x1F48,0x1F4D) || inRange(c, 0x1F50,0x1F57)
		|| inRange(c, 0x1F59)        || inRange(c, 0x1F5B)        || inRange(c, 0x1F5D) 
		|| inRange(c, 0x1F5F,0x1F7D) || inRange(c, 0x1F80,0x1FB4) || inRange(c, 0x1FB6,0x1FBC) 
		|| inRange(c, 0x1FBE)        || inRange(c, 0x1FC2,0x1FC4) || inRange(c, 0x1FC6,0x1FCC) 
		|| inRange(c, 0x1FD0,0x1FD3) || inRange(c, 0x1FD6,0x1FDB) || inRange(c, 0x1FE0,0x1FEC) 
		|| inRange(c, 0x1FF2,0x1FF4) || inRange(c, 0x1FF6,0x1FFC) || inRange(c, 0x2126) 
		|| inRange(c, 0x212A,0x212B) || inRange(c, 0x212E)        || inRange(c, 0x2180,0x2182) 
		|| inRange(c, 0x3041,0x3094) || inRange(c, 0x30A1,0x30FA) || inRange(c, 0x3105,0x312C) 
		|| inRange(c, 0xAC00,0xD7A3)
		) return TRUE;
	return FALSE;
}


bool isIdeographic(const QChar &c) {
	if (
		inRange(c, 0x4E00,0x9FA5) || inRange(c, 0x3007) || inRange(c, 0x3021,0x3029)
		) return TRUE;
	return FALSE;
}


bool inRange(const QChar &c, int lower, int upper) {
	if (upper < 0) return c == QChar(lower);
	return c >= QChar(lower) && c <= QChar(upper);
}


QString encodeToXml(const QString &str) {
	QString xml;
	QChar currChar;
	for (int i = 0; i < str.length(); i++) {
		currChar = str.at(i);
		switch (currChar.toAscii()) {
			case '&': xml += "&amp;"; break;
			case '<': xml += "&lt;"; break;
			case '>': xml += "&gt;"; break;
			case '\'': xml += "&apos;"; break;
			case '\"': xml += "&quot;"; break;
			default: xml += currChar; break;
		}
	}
	return xml;
}


QString encodeComment(const QString &str) {
	QString xml;
	QChar currChar;
	for (int i = 0; i < str.length(); i++) {
		currChar = str.at(i);
		switch (currChar.toAscii()) {
			case '-': xml += "~"; break;
			default: xml += currChar; break;
		}
	}
	return xml;
}
