/**************************************************************************
*   Copyright (C) 2005-2006 by Jinsoo Kang                                *
*   jsk@gravitybind.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
**************************************************************************/

#include "tx.cfg"
#include "xmltable.h"
#include "xmltableitem.h"

#include <iostream>
using namespace std;


XmlTableItem::NodeType XmlTable::itemType(int r, int c) {
	if (c < 0 || r < 0) return XmlTableItem::EMPTY;
	XmlTableItem* currXmlItem = xmlItem(r, c);
	if (currXmlItem == NULL) {
		return XmlTableItem::EMPTY;
	}
	return currXmlItem->itemType();
}


void XmlTableItem::decreaseDepth() {
	hierDepth--;
	if (hierDepth < 1) hierDepth = 1;
}


void XmlTableItem::increaseDepth() {
	hierDepth++;	
}


int XmlTableItem::depth() {
	if (itemType() == EMPTY) return ((nextDepth() < previousDepth()) ? nextDepth() : previousDepth());
	return hierDepth;
}


XmlTableItem::NodeType XmlTable::itemTypeForString(const QString & str) {
	for (int i = XmlTableItem::EMPTY; i <= XmlTableItem::COMMENT; i++) {
		if (str == style((XmlTableItem::NodeType)i)->name()) return (XmlTableItem::NodeType)i;
	}
	return XmlTableItem::EMPTY;
}


XmlTableItem::XmlTableItem(NodeType type, Q3Table *t, EditType et, const QString &txt )
: Q3TableItem( t, et ) {
	setItemType(type);
	QString rtxt = txt;
	//rtxt.replace(QChar('\t'), QChar(' '));
	setText(rtxt);
	setSizeHint(TRUE);
	setDepth(0);
	setExpanded(true);
}


void XmlTableItem::setItemType(NodeType type) {
	this->type = type;
	setWordWrap(XmlTable::style(type)->isWordWrap());
}


int XmlTableItem::alignment() const {
	return XmlTable::style(type)->alignment();
}


int XmlTableItem::spanWidth() const {
	int retval = 0;
	for (int currCol = col(), i = 0; i < colSpan(); i++, currCol++) {
		retval += table()->columnWidth(currCol);
	}
	return retval;
}



int XmlTableItem::countWrapedLines(const QString & str, int maxWidth, const QFontMetrics & metric, int wordWrap) const {
	int h = 1;
	int currWidth = 0;
	int lineIndex = 0;
	int wordWidth = 0;
	int charWidth = 0;
	QChar currChar;
	static int i = 0;
	static int limit = 0;
	static int j = 0;

	for (i = limit = 0; i < str.length() && limit < 1000; i++, limit++) {
		currChar = str.at(i);
		if (currChar == '\n') {//new line
			h++;
			currWidth = 0;
			lineIndex = i;
			continue;
		}
		charWidth = metric.charWidth(str, i);
		currWidth += charWidth;
		if (currChar.isSpace()) wordWidth = 0;
		else wordWidth += charWidth;

		if (currWidth < maxWidth) continue;

		if (currChar.isSpace()) {//space at end of line -> absorb remaining spaces
			do {
				i++;
				if (i < str.length()) currChar = str.at(i);
			} while (i < str.length() && currChar.isSpace());
			h++;
			currWidth = 0;
			lineIndex = i;
		}
		else {//character at end of line
			j = i;
			if (wordWrap == Qt::TextWrapAnywhere && wordWidth >= maxWidth) {//break word
				do {
					j--;
					currChar = str.at(j);
					charWidth = metric.charWidth(str, j);
					wordWidth -= charWidth;
				} while (!currChar.isSpace() && wordWidth >= maxWidth);
			}
			else {//go back to last space
				do {
					j--;
					if (j > lineIndex) currChar = str.at(j);
				} while (!currChar.isSpace() && j > lineIndex);
			}
			if (j > lineIndex) {
				i = j;
				h++;
				currWidth = 0;
				wordWidth = 0;
				lineIndex = i;
			}
		}
	}
	return h;
}


QSize XmlTableItem::sizeHint() const {
	return size;
}


void XmlTableItem::setSizeHint(bool isInit) {
	QFontMetrics metric(XmlTable::style(type)->font());
	if (isInit) {
		size.setWidth( 100 );
		size.setHeight( metric.height()+1 );
		return;
	}
	QString val = text();

	int w = XmlTable::minColumnWidth, h = 0;
	int oneCharWidth = metric.width('9');

	if (XmlTable::style(type)->isWordWrap()) {
		int maxWidth = (spanWidth() > (oneCharWidth*5)) ? spanWidth() : (oneCharWidth*5);
		h = countWrapedLines( 
			val, maxWidth-2, metric, XmlTable::style(type)->wordWrap() 
			);
	}
	else {
		h = 1;
		QStringList slist = QStringList::split('\n', val, TRUE);
		QStringList::iterator it;
		int currWidth = 0;
		for ( it = slist.begin(); it != slist.end(); ++it ) {
			currWidth += metric.width(*it, (*it).length());
		}
		currWidth += oneCharWidth;
		if (currWidth > w) w = currWidth;
	}
	int dh = metric.lineSpacing();

	h = h * dh;
	if (type == EMPTY) {
		if (w < XmlTable::minColumnWidth) w = XmlTable::minColumnWidth;
	}
	if (type == ELEMENT) {
		int dh = XmlTable::indentWidth;
		size.setWidth(w + dh*(hierDepth-1) + 3);
	}
	else size.setWidth(w + 3);
	size.setHeight(h);
}


void XmlTableItem::setDepth(int d) {
	hierDepth = d+1;
}


void XmlTableItem::setExpanded(bool isExpanded) {
	this->isExpanded = isExpanded;
}


bool XmlTableItem::expanded() {
	return this->isExpanded;
}


int XmlTableItem::nextDepth() {
	XmlTable* xtable = (XmlTable*) table();
	for (int r = row()+1; r <= xtable->maxRows(); r++) {
		XmlTableItem* itemBelow = xtable->xmlItem(r, col());
		if (itemBelow != NULL && itemBelow->itemType() == ELEMENT) {
			return itemBelow->depth();
		}
	}
	return 0;
}


int XmlTableItem::previousDepth() {
	XmlTable* xtable = (XmlTable*) table();
	for (int r = row()-1; r >= 0; r--) {
		XmlTableItem* itemAbove = xtable->xmlItem(r, col());
		if (itemAbove != NULL && itemAbove->itemType() == ELEMENT) {
			return itemAbove->depth();
		}
	}
	return 0;
}


bool XmlTableItem::isSpanCell() const {
	switch (type) {
		case TEXT:
		case CDATA:
		case COMMENT:
			return TRUE;
		default:
			return FALSE;
	}
}


QWidget* XmlTableItem::createEditor() const {
	XmlTable *xtable = (XmlTable*)table();
	xtable->setEditingLocation(row(), col());
	switch (type) {
		case EMPTY:
		case ATTR_KEY:
		case ATTR_VAL:
		case ELEMENT: {
			QLineEdit* w = (QLineEdit*) Q3TableItem::createEditor();
			w->setPaletteBackgroundColor(XmlTable::style(type)->bgColor());
			w->setAlignment((Qt::Alignment) alignment());
			w->setFont(XmlTable::style(type)->font());
			return w;
					  }
		case TEXT:
		case CDATA:
		case COMMENT: {
			Q3TextEdit *textEdit = new Q3TextEdit(table()->viewport());
			textEdit->setText(text());
			textEdit->setWordWrap(
				XmlTable::style(type)->isWordWrap() ? 
				Q3TextEdit::WidgetWidth : Q3TextEdit::NoWrap
				);
			textEdit->setPaletteBackgroundColor(XmlTable::style(type)->bgColor());
			textEdit->setAlignment(alignment());
			textEdit->setFont(XmlTable::style(type)->font());
			textEdit->setTabStopWidth(4);
			textEdit->setTextFormat(Qt::PlainText);
			textEdit->setHScrollBarMode(Q3ScrollView::AlwaysOff);
			textEdit->setLineWidth(0);
			textEdit->setMidLineWidth(0);
			return textEdit;
					  }
		default:
			return Q3TableItem::createEditor();
	}
}


void XmlTableItem::setContentFromEditor( QWidget *w ) {
	bool modified;
	XmlTable* xtable = (XmlTable*)table();
	if (w->inherits("Q3TextEdit")) {
		Q3TextEdit* ed = (Q3TextEdit*) w;
		QString oldstr = text();
		QString newstr = ed->text();
		if (oldstr.isNull()) oldstr = "";
		if (newstr.isNull()) newstr = "";
		if (oldstr != newstr) {
			xtable->addUndoRecord(row(), col(), row(), col());
		}
		setText(ed->text());
		modified = ed->isModified();
	}
	else {
		QLineEdit* ed = (QLineEdit*) w;
		QString oldstr = text();
		QString newstr = ed->text();
		if (oldstr.isNull()) oldstr = "";
		if (newstr.isNull()) newstr = "";
		if (oldstr != newstr) {
			xtable->addUndoRecord(row(), col(), row(), col());
		}
		modified = ed->isModified();
		Q3TableItem::setContentFromEditor(w);
	}
	if (modified) {
		xtable->setModified(TRUE);
		xtable->adjustRow(row());
		xtable->adjustColumn(col());
	}
}


XmlTableItem::NodeType XmlTableItem::itemType() const {
	return type;
}

