/**************************************************************************
*   Copyright (C) 2005-2006 by Jinsoo Kang                                *
*   jsk@gravitybind.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "xmltablestyle.h"



XmlTableStyle::XmlTableStyle() {
}


QString XmlTableStyle::fontName() {
	QString retval = QString("%1 %2")
		.arg(fontVal.family())
		.arg(fontVal.pointSize());
	return retval;
}


const QString & XmlTableStyle::name() const {
	return nameVal;
}


const QFont & XmlTableStyle::font() const {
	return fontVal;
}


const QColor & XmlTableStyle::bgColor() const {
	return bgColorVal;
}


const QColor & XmlTableStyle::fgColor() const {
	return fgColorVal;
}


int XmlTableStyle::alignment() const {
	return align;
}


int XmlTableStyle::wordWrap() const {
	return wrap;
}


bool XmlTableStyle::isWordWrap() {
	return wrap == Qt::TextWordWrap || wrap == Qt::TextWrapAnywhere;
}


QString XmlTableStyle::alignText() {
	if (align & Qt::AlignLeft) return QObject::tr("left");
	else if (align & Qt::AlignRight) return QObject::tr("right");
	else if (align & Qt::AlignHCenter) return QObject::tr("center");
	else return QObject::tr("left");
}


QString XmlTableStyle::valignText() {
	if (align & Qt::AlignTop) return QObject::tr("top");
	else if (align & Qt::AlignBottom) return QObject::tr("bottom");
	else if (align & Qt::AlignVCenter) return QObject::tr("middle");
	else return QObject::tr("middle");
}


QString XmlTableStyle::wordWrapText() {
	if (wrap & Qt::TextSingleLine) return QObject::tr("single line");
	else if (wrap & Qt::TextWrapAnywhere) return QObject::tr("break anywhere");
	else if (wrap & Qt::TextWordWrap) return QObject::tr("break on words");
	else return QObject::tr("break anywhere");
}


void XmlTableStyle::setName(const QString & nameVal) {
	this->nameVal = nameVal;
}


void XmlTableStyle::setFont(const QString & fontVal) {
	this->fontVal.fromString(fontVal);
}


void XmlTableStyle::setFont(const QFont & fontVal) {
	this->fontVal = fontVal;
}


void XmlTableStyle::setBgColor(const QString & bg) {
	bgColorVal.setNamedColor(bg);
}


void XmlTableStyle::setFgColor(const QString & fg) {
	fgColorVal.setNamedColor(fg);
}


void XmlTableStyle::setAlignment(int align) {
	this->align = align;
}


void XmlTableStyle::setWordWrap(int wrap) {
	this->wrap = wrap;
}


int XmlTableStyle::textFlags() const {
	return alignment() | wordWrap();
}

